import QtQuick 2.9
import org.kde.plasma.core 2.1
import QtQuick.Layouts 1.15
Item {
    id: dashboard
    width:480

    property var osInfo:{}
    property var url2:"/Data/osInfo.json"
    property var smartStatusFailed:false
    property var lowDiskSpaceWarning:false
    property var tempWarning:false
    property var lowMemWarning:false


    Component.onCompleted: {
        getData(url2)
    }

    Timer {
        interval:1500
        running:true
        repeat:false
        onTriggered: checkSmartStatus()
    }

    function getData(url){
        var xhr = new XMLHttpRequest;
        xhr.open("GET", url); // set Method and File
        xhr.onreadystatechange = function () {
            if(xhr.readyState === XMLHttpRequest.DONE){ // if request_status == DONE
                var response = xhr.responseText;
                osInfo = JSON.parse(response)
            }
        }
        xhr.send(); // begin the request
    }


    ColumnLayout {
        id:c1
        spacing:2
        anchors.leftMargin:10

        Rectangle {
            id:title
            color:"transparent"
            border.color:"gray"
            radius:8
            implicitHeight:125
            width:dashboard.width*.995

            Image {
                id:logo
                source:"tux.png"
                //width:32
                height :64
                fillMode:Image.PreserveAspectFit
                smooth:true
                anchors.bottom:parent.bottom
                anchors.right:parent.right
            }
        }

        Text {
            id:t1
            text:osInfo.hostname //"Digi-Boxx"
            color:Theme.textColor
            anchors.top:title.top
            anchors.topMargin:10
            font.capitalization: Font.Capitalize
            anchors.horizontalCenter:title.horizontalCenter
            antialiasing : true
            font.pointSize:12
        }

        Text {
            id:osInfo1
            text:"OS:  "+osInfo.distro+ " "+osInfo.code+"  "+osInfo.release
            color:Theme.textColor
            anchors.topMargin:5
            antialiasing:true
            anchors.horizontalCenter:title.horizontalCenter;
            anchors.top:t1.bottom
            font.pointSize:11
        }

        Text {
            id:osInfo2
            text:"Kernel:  "+osInfo.kernel
            color:Theme.textColor
            anchors.topMargin:5
            antialiasing:true
            anchors.horizontalCenter:osInfo1.horizontalCenter;
            anchors.top:osInfo1.bottom
            font.pointSize:11
        }

        Text {
            id:utime
            text:"Uptime: "+secondsToDhms(dataSource.data[dataSource.uptime].value)
            color:Theme.textColor
            anchors.top:osInfo2.bottom
            anchors.horizontalCenter:title.horizontalCenter
            antialiasing : true
            font.pointSize:11
        }


        Text {
            id:osMsg   // system error messages
            text:""
            color:"red"
            anchors.top:utime.bottom
            anchors.horizontalCenter:title.horizontalCenter
            antialiasing : true
            font.pointSize:12
            anchors.topMargin:5
            visible:false
        }

        Rectangle {
            id:info
            color:"transparent"
            border.color:"gray"
            radius:8
            height:60
            width:dashboard.width*.995
            anchors.top:title.bottom
            anchors.topMargin:10
        }
        Row {
            anchors.top:info.top
            anchors.topMargin:20
            anchors.left:c1.left
            anchors.leftMargin:10
            spacing:20
            Image {
                source:"utilities-system-monitor-symbolic.png"
                height:32
                width:32
                smooth:true
                anchors.topMargin:-5
                anchors.top:parent.top
            }
            Text  {
                id: infotext2
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                width:80
                text:tempLoad()

                Rectangle {
                    id:tempRect
                    anchors.top:parent.bottom
                    anchors.topMargin:5
                    width:80
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:tempRect.top
                    anchors.left:tempRect.left
                    width:(tempRect.width)*Math.round(cToF(dataSource.data[dataSource.temp].value))/240
                    height:5
                    radius:8
                    color:sysLoadColors(Math.round(cToF(dataSource.data[dataSource.temp].value))/240*100)
                }
            }


            Text  {
                id: infotext1
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text:memLoad();
                Rectangle {
                    id:memRect
                    anchors.top:infotext1.bottom
                    anchors.topMargin:5
                    width:80
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:memRect.top
                    anchors.left:memRect.left
                    width:memRect.width*((dataSource.data[dataSource.memAvailable].value/1024/1024).toFixed(1)/15.4).toFixed(2)
                    height:5
                    radius:8
                    color:sysLoadColors(100-((dataSource.data[dataSource.memAvailable].value/1024/1024).toFixed(1)/15.4).toFixed(2)*100)
                }
            }

            Text  {
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text: cpuLoad()
                Rectangle {
                    id:cpuRect
                    anchors.top:parent.bottom
                    anchors.left:parent.left
                    width:80
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:cpuRect.top
                    anchors.left:cpuRect.left
                    width:cpuRect.width*(Math.round(dataSource.data[dataSource.averageClock].value)/4000)
                    height:5
                    radius:8
                    color:sysLoadColors((dataSource.data[dataSource.averageClock].value/4000).toFixed(2)*100)
                }
            }
        }

        Rectangle {   /// System Fan Sensors
            id:info2
            color:"transparent"
            border.color:"gray"
            radius:8
            height:60
            width:dashboard.width*.995
            anchors.top:info.bottom
            anchors.topMargin:10
        }

        Row {
            anchors.top:info2.top
            anchors.topMargin:20
            anchors.left:c1.left
            anchors.leftMargin:10
            spacing:25

            Image {
                source:"sensors-fan-symbolic.png"
                height:32
                width:32
                smooth:true
                anchors.topMargin:-5
                anchors.top:parent.top
            }
            Text  {
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text:"Front Fan: "+Math.round(dataSource.data[dataSource.frontFan].value)

                Rectangle {
                    id:frontFanRect
                    anchors.top:parent.bottom
                    width:90
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:frontFanRect.top
                    anchors.left:fronFanRect.left
                    width:frontFanRect.width*((dataSource.data[dataSource.frontFan].value)/1000).toFixed(2)
                    height:5
                    radius:8
                    color:sysLoadColors(1-(dataSource.data[dataSource.frontFan].value/1000*100).toFixed(0))
                }
            }

            Text  {
                id: infotext4
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text:"CPU Fan: "+Math.round(dataSource.data[dataSource.cpuFan].value)

                Rectangle {
                    id:cpuFanRect
                    anchors.top:infotext4.bottom
                    width:90
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:cpuFanRect.top
                    anchors.left:cpuFanRect.left
                    width:cpuFanRect.width*((dataSource.data[dataSource.cpuFan].value)/3200).toFixed(2)
                    height:5
                    radius:8
                    color:sysLoadColors((dataSource.data[dataSource.cpuFan].value/3200*100).toFixed(0))
                }
            }

            Text  {
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text:"Exhaust Fan: "+Math.round(dataSource.data[dataSource.exhaustFan].value)

                Rectangle {
                    id:caseFanRect
                    anchors.top:parent.bottom
                    width:90
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:caseFanRect.top
                    anchors.left:caseFanRect.left
                    //width:caseFanRect.width*(1-(dataSource.data[dataSource.exhaustFan].value)/1600).toFixed(0)
                    width:caseFanRect.width*(dataSource.data[dataSource.exhaustFan].value/1600).toFixed(2)
                    height:5
                    radius:8
                    color:sysLoadColors((dataSource.data[dataSource.exhaustFan].value/1600*100).toFixed(0))
                }
            }
        }

        Rectangle {
            id:info3            /// Network Status
            color:"transparent"
            border.color:"gray"
            radius:8
            height:60
            width:dashboard.width*.995
            anchors.top:info2.bottom
            anchors.topMargin:10
        }

        Row {
            anchors.top:info3.top
            anchors.topMargin:20
            anchors.left:c1.left
            anchors.leftMargin:10
            spacing:25
            Image {
                source:"network-wired-symbolic.png"
                height:36
                width:36
                smooth:true
                anchors.topMargin:-5
                anchors.top:parent.top
            }

            Text  {
                id: infotext6
                color:Theme.textColor
                font.pointSize:11
                leftPadding:-10
                antialiasing : true
                text: "Firewall:"

                Rectangle {
                    anchors.left:infotext6.right
                    anchors.leftMargin:10
                    anchors.top:infotext6.top
                    anchors.topMargin:6
                    height:12
                    width:12
                    radius:180
                    color:(osInfo.firewall==="active") ? "green" : "red"
                    smooth:true
                }
            }

            Text  {
                id: infotext7
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                leftPadding:15
                text:"Local:  "+osInfo.intip
            }

            Text  {
                id: infotext8
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text:"iNet:  "+osInfo.extip
            }

        }


        Rectangle {
            id:info4        //// drive data
            color:"transparent"
            border.color:"gray"
            radius:8
            height:60
            width:dashboard.width*.995
            anchors.top:info3.bottom
            anchors.topMargin:10
        }

        Row {
            anchors.top:info4.top
            anchors.topMargin:20
            anchors.left:c1.left
            anchors.leftMargin:10
            spacing:12


            Image {
                source:"drive-harddisk-system-symbolic.png"
                height:36
                width:36
                smooth:true
                anchors.topMargin:-5
                anchors.top:parent.top
            }

            Text  {
                id: i1   // free drive space
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text: "System: "+fsUsedLoad(dataSource.data[dataSource.fileSystemUsed].value,dataSource.data[dataSource.fileSystemFree].value)

                Rectangle {
                    id:fsSystemRect
                    anchors.top:i1.bottom
                    width:infotext2.width
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    id:r2
                    anchors.top:fsSystemRect.top
                    anchors.left:fsSystemRect.left
                    width:fsSystemRect.width*(dataSource.data[dataSource.fileSystemLevel].value/100)
                    height:5
                    radius:8
                    color:sysLoadColors(dataSource.data[dataSource.fileSystemLevel].value)

                }
            }

            Text  {
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text: "Data: "+fsUsedLoad(dataSource.data[dataSource.fileSystemDataUsed].value,dataSource.data[dataSource.fileSystemDataFree].value)

                Rectangle {
                    id:fsDataRect
                    anchors.top:parent.bottom
                    width:infotext2.width
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:fsDataRect.top
                    anchors.left:fsDataRect.left
                    width:fsDataRect.width*(dataSource.data[dataSource.fileSystemDataLevel].value/100)
                    height:5
                    radius:8
                    color:sysLoadColors(dataSource.data[dataSource.fileSystemDataLevel].value)
                }
            }

            Text  {
                id: i3
                color:Theme.textColor
                font.pointSize:11
                antialiasing : true
                text: "Backup: "+fsUsedLoad(dataSource.data[dataSource.fileSystemBackupUsed].value,dataSource.data[dataSource.fileSystemBackupFree].value)

                Rectangle {
                    id:fsBackupRect
                    anchors.top:i3.bottom
                    width:infotext2.width
                    anchors.topMargin:5
                    height:5
                    radius:8
                    color:"#2e343e"
                    border.color:"#323843"
                }

                Rectangle {
                    anchors.top:fsBackupRect.top
                    anchors.left:fsBackupRect.left
                    width:fsBackupRect.width*(dataSource.data[dataSource.fileSystemBackupLevel].value/100)
                    height:5
                    radius:8
                    color:sysLoadColors(dataSource.data[dataSource.fileSystemBackupLevel].value)

                }
            }
        }
    }


    function checkSmartStatus() {

        if (osInfo.sda != "PASSED") {
            root.panel_icon = "SuggestionError"
            root.tooltext="Device /dev/sda failed SMART tests!"
            osMsg.text="Device /dev/sda failed SMART tests!"
            osMsg.visible=true
            smartStatusFailed=true
        }
        else if (osInfo.sdb != "PASSED") {
            root.panel_icon= "SuggestionError"
            root.tooltext="Device /dev/sdb failed SMART tests!"
            osMsg.text="Device /dev/sdb failed SMART tests!"
            osMsg.visible=true
            smartStatusFailed=true
        }
        else if (osInfo.sdc != "PASSED") {
            root.panel_icon="SuggestionError"
            root.tooltext="Device /dev/sdc failed SMART tests!"
            osMsg.text="Device /dev/sdc failed SMART tests!"
            osMsg.visible=true
            smartStatusFailed=true
        }

        else if (osInfo.firewall!="active") {
            root.panel_icon="SuggestionError"
            root.tooltext="Firewall Not Active!"
            osMsg.text=="Firewall Not Active!"
            osMsg.visible=true
            smartStatusFailed=true
        }

        else if (smartStatusFailed===true){
            smartStatusFailed=false
            root.panel_icon="view-process-system"
            root.tooltext="All Systems Go!"
            osMsg.text=""
            osMsg.visible=false
        }
    }


    function sysLoadColors(x) {
        if (x < 71) { // 0..70
            return "green" }
            else if ( x > 70 && x < 86) { //71..85
                return "yellow" }
                else if ( x > 85 && x < 96 ) { //86..95
                    return "orange" }
                    else if ( x > 96) { //95..100
                        return "red"
                    }
    }

    function cToF(celsius)
    {
        var cTemp = celsius;
        var cToFahr = cTemp * 9 / 5 + 32;
        return cToFahr;
    }

    function cpuLoad() {
        var cpuLoadPart = "";
        var cpuClockPart = "";
        var cpuLoad = dataSource.data[dataSource.totalLoad] ? dataSource.data[dataSource.totalLoad].value : 0
        var averageClock = dataSource.data[dataSource.averageClock] ? dataSource.data[dataSource.averageClock].value : 0
        cpuLoadPart = i18n("CPU: %1%", Math.round(cpuLoad))
        cpuClockPart = i18n("%1 GHz", Number.parseFloat((averageClock/1000).toFixed(2)))
        var cpuPart = cpuLoadPart+"~"+cpuClockPart
        return cpuPart
    }


    function memLoad() {
        var memFree = parseFloat(dataSource.data[dataSource.memFree] ? dataSource.data[dataSource.memFree].value : 0) / 1024
        var memUsed = parseFloat(dataSource.data[dataSource.memUsed] ? dataSource.data[dataSource.memUsed].value : 0) / 1024
        var memApplication = parseFloat(dataSource.data[dataSource.memApplication] ? dataSource.data[dataSource.memApplication].value : 0) / 1024
        var memAvailable = parseFloat(dataSource.data[dataSource.memAvailable] ? dataSource.data[dataSource.memAvailable].value : 0) / 1024
        var memTotal = memFree + memUsed
        var ramFree =(dataSource.data[dataSource.memAvailable].value) - (dataSource.data[dataSource.memApplication].value)
        //var memoryPart = i18n("RAM: %1/%2 GiB", ((memAvailable-memApplication)/1024).toFixed(1), (Math.round(memTotal)/1000).toFixed(0))
        var memoryPart = i18n("RAM: %1/%2 GiB",(dataSource.data[dataSource.memAvailable].value/1024/1024).toFixed(1), (Math.round(memTotal)/1000).toFixed(0))

        if ((dataSource.data[dataSource.memAvailable].value/1024/1024).toFixed(1) < 2) { /// change this for low RAM warning less than 2gb
            root.panel_icon="SuggestionError";
            root.tooltext="Low RAM Detected!";
            osMsg.text="Low RAM Detected!";
            osMsg.visible=true;
            lowMemWarning=true; }

            else if (lowMemWarning) {
                lowMemWarning=false;
                root.panel_icon="view-process-system";
                root.tooltext="All Systems Go!";
                osMsg.text="";
                osMsg.visible=false;
            }
            return memoryPart
    }

    function tempLoad() {
        var temp1 = dataSource.data[dataSource.temp] ? dataSource.data[dataSource.temp].value : 0
        var impTemp = Math.round(cToF(temp1))
        var tempPart = "Temp:  "+impTemp+"°F"

        if (impTemp > 240) {      /// change this for high temp warning more than 240
            root.panel_icon="SuggestionError";
            root.tooltext="OverHeating Detected!";
            osMsg.text="OverHeating Detected!";
            osMsg.visible=true
            tempWarning=true;
        }
        else if  (tempWarning){ // reset warnings
            tempWarning=false;
            root.panel_icon="view-process-system";
            root.tooltext="All Systems Go!";
            osMsg.text="";
            osMsg.visible=false;
        }
        return tempPart
    }

    function fsUsedLoad(x,y) {
        var fsUsed = x ? x : 0
        var fsFree = y ? y : 0

        var fsLevel = dataSource.data[dataSource.fileSystemLevel] ? dataSource.data[dataSource.fileSystemLevel].value : 0
        var fsTotal = dataSource.data[dataSource.fileSystemTotal] ? dataSource.data[dataSource.fileSystemTotal].value : 0
        var fsCritical = (.1*Math.round(Number.parseFloat(((fsTotal/1024)/1024)))).toFixed(2)
        var fsPart = i18n("%1/%2 GB", Math.round(Number.parseFloat(((fsUsed/1024)/1024).toFixed(2))), Math.round(Number.parseFloat(((fsFree/1024)/1024).toFixed(2))))

        if (Math.round(y/1024/1024) < 10) {  // less than 10gb free
            root.panel_icon= "SuggestionError"
            root.tooltext="Low Disk Space Detected!"
            osMsg.text="Low Disk Space Detected!"
            lowDiskSpaceWarning=true
        }
        else  if (lowDiskSpaceWarning) {
            lowDiskSpaceWarning=false
            root.panel_icon="view-process-system"
            root.tooltext="All Systems Go!"
            osMsg.text=""
            osMsg.visible=false
        }
        return fsPart
    }

    function secondsToDhms(seconds) {   // used for uptime
        seconds = Number(seconds);
        var d = Math.floor(seconds / (3600*24));
        var h = Math.floor(seconds % (3600*24) / 3600);
        var m = Math.floor(seconds % 3600 / 60);
        var s = Math.floor(seconds % 60);

        var dDisplay = d > 0 ? d + (d == 1 ? " day, " : " days, ") : "";
        var hDisplay = h > 0 ? h + (h == 1 ? " hour, " : " hours, ") : "";
        var mDisplay = m > 0 ? m + (m == 1 ? " minute" : " minutes") : "";
        var sDisplay = s > 0 ? s + (s == 1 ? " second" : " seconds") : "";
        return dDisplay + hDisplay + mDisplay// + sDisplay;
    }

    DataSource {
        id: dataSource
        engine: "systemmonitor"

        readonly property string cpuSystem: "cpu/system/"
        readonly property string uptime:"system/uptime"
        readonly property string temp: "lmsensors/coretemp-isa-0000/Package_id_0"

        readonly property string frontFan:"lmsensors/nct6791-isa-0290/fan5"
        readonly property string cpuFan:"lmsensors/nct6791-isa-0290/fan2"
        readonly property string exhaustFan:"lmsensors/nct6791-isa-0290/fan1"

        readonly property string averageClock: cpuSystem + "AverageClock"
        readonly property string totalLoad: cpuSystem + "TotalLoad"

        readonly property string memPhysical: "mem/physical/"
        readonly property string memFree: memPhysical + "free"
        readonly property string memApplication: memPhysical + "application"
        readonly property string memAvailable: memPhysical + "available"
        readonly property string memUsed: memPhysical + "used"

        readonly property string fileSystemUsed: "partitions/__root__/usedspace"
        readonly property string fileSystemFree: "partitions/__root__/freespace"
        readonly property string fileSystemLevel:"partitions/__root__/filllevel"

        readonly property string fileSystemDataUsed: "partitions/Data/usedspace"
        readonly property string fileSystemDataFree: "partitions/Data/freespace"
        readonly property string fileSystemDataLevel:"partitions/Data/filllevel"

        readonly property string fileSystemBackupUsed: "partitions/Backup/usedspace"
        readonly property string fileSystemBackupFree: "partitions/Backup/freespace"
        readonly property string fileSystemBackupLevel:"partitions/Backup/filllevel"

        interval: 1000
        connectedSources:[cpuSystem,uptime,memFree,memApplication,memAvailable,memUsed,averageClock,totalLoad,temp,fileSystemUsed,fileSystemFree,fileSystemLevel,fileSystemDataLevel,fileSystemBackupLevel,frontFan,cpuFan,exhaustFan,fileSystemDataUsed,fileSystemDataFree,fileSystemBackupUsed,fileSystemBackupFree]
    }
}
